@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * Almanah
- * Copyright (C) Philip Withnall 2008 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2008-2009 <philip@tecnocode.co.uk>
  * 
  * Almanah is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,14 @@
 #include "main.h"
 #include "main-window.h"
 
-static void almanah_search_dialog_init (AlmanahSearchDialog *self);
 static void sd_response_cb (GtkDialog *dialog, gint response_id, AlmanahSearchDialog *search_dialog);
 static void sd_results_selection_changed_cb (GtkTreeSelection *tree_selection, GtkWidget *button);
 
+/* GtkBuilder callbacks */
+void sd_search_button_clicked_cb (GtkButton *self, AlmanahSearchDialog *search_dialog);
+void sd_results_tree_view_row_activated_cb (GtkTreeView *self, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data);
+void sd_view_button_clicked_cb (GtkButton *self, AlmanahSearchDialog *search_dialog);
+
 struct _AlmanahSearchDialogPrivate {
 	GtkEntry *sd_search_entry;
 	GtkListStore *sd_results_store;
@@ -66,9 +70,9 @@ almanah_search_dialog_new (void)
 	GError *error = NULL;
 	const gchar *interface_filename = almanah_get_interface_filename ();
 	const gchar *object_names[] = {
-		"dry_search_dialog",
-		"dry_sd_search_button_image",
-		"dry_sd_results_store",
+		"almanah_search_dialog",
+		"almanah_sd_search_button_image",
+		"almanah_sd_results_store",
 		NULL
 	};
 
@@ -92,7 +96,7 @@ almanah_search_dialog_new (void)
 	}
 
 	gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
-	search_dialog = ALMANAH_SEARCH_DIALOG (gtk_builder_get_object (builder, "dry_search_dialog"));
+	search_dialog = ALMANAH_SEARCH_DIALOG (gtk_builder_get_object (builder, "almanah_search_dialog"));
 	gtk_builder_connect_signals (builder, search_dialog);
 
 	if (search_dialog == NULL) {
@@ -103,17 +107,14 @@ almanah_search_dialog_new (void)
 	priv = ALMANAH_SEARCH_DIALOG (search_dialog)->priv;
 
 	/* Grab our child widgets */
-	priv->sd_search_entry = GTK_ENTRY (gtk_builder_get_object (builder, "dry_sd_search_entry"));
-	priv->sd_results_store = GTK_LIST_STORE (gtk_builder_get_object (builder, "dry_sd_results_store"));
-	priv->sd_results_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_builder_get_object (builder, "dry_sd_results_tree_view")));
+	priv->sd_search_entry = GTK_ENTRY (gtk_builder_get_object (builder, "almanah_sd_search_entry"));
+	priv->sd_results_store = GTK_LIST_STORE (gtk_builder_get_object (builder, "almanah_sd_results_store"));
+	priv->sd_results_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_builder_get_object (builder, "almanah_sd_results_tree_view")));
 
 	g_signal_connect (priv->sd_results_selection, "changed", G_CALLBACK (sd_results_selection_changed_cb),
-			  gtk_builder_get_object (builder, "dry_sd_view_button"));
+			  gtk_builder_get_object (builder, "almanah_sd_view_button"));
 
-	gtk_widget_grab_default (GTK_WIDGET (gtk_builder_get_object (builder, "dry_sd_search_button")));
-
-	/* Prettify the UI */
-	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "dry_sd_results_label")));
+	gtk_widget_grab_default (GTK_WIDGET (gtk_builder_get_object (builder, "almanah_sd_search_button")));
 
 	g_object_unref (builder);
 
@@ -141,7 +142,7 @@ sd_search_button_clicked_cb (GtkButton *self, AlmanahSearchDialog *search_dialog
 {
 	GDate *results;
 	gint result_count;
-	guint i;
+	gint i;
 	GtkTreeIter iter;
 	AlmanahSearchDialogPrivate *priv = search_dialog->priv;
 
