@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * Almanah
- * Copyright (C) Philip Withnall 2008 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2008-2009 <philip@tecnocode.co.uk>
  * 
  * Almanah is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,8 +33,10 @@
 #include "entry.h"
 #include "definition.h"
 #include "storage-manager.h"
+#include "almanah-marshal.h"
+
+#define ENCRYPTED_SUFFIX ".encrypted"
 
-static void almanah_storage_manager_init (AlmanahStorageManager *self);
 static void almanah_storage_manager_finalize (GObject *object);
 static void almanah_storage_manager_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void almanah_storage_manager_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -52,6 +54,7 @@ enum {
 enum {
 	SIGNAL_DISCONNECTED,
 	SIGNAL_DEFINITION_ADDED,
+	SIGNAL_DEFINITION_MODIFIED,
 	SIGNAL_DEFINITION_REMOVED,
 	LAST_SIGNAL
 };
@@ -88,14 +91,20 @@ almanah_storage_manager_class_init (AlmanahStorageManagerClass *klass)
 				G_TYPE_FROM_CLASS (klass),
 				G_SIGNAL_RUN_LAST,
 				0, NULL, NULL,
-				g_cclosure_marshal_VOID__OBJECT,
-				G_TYPE_NONE, 1, ALMANAH_TYPE_STORAGE_MANAGER);
+				almanah_marshal_VOID__STRING_STRING,
+				G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
 	storage_manager_signals[SIGNAL_DEFINITION_ADDED] = g_signal_new ("definition-added",
 				G_TYPE_FROM_CLASS (klass),
 				G_SIGNAL_RUN_LAST,
 				0, NULL, NULL,
 				g_cclosure_marshal_VOID__OBJECT,
 				G_TYPE_NONE, 1, ALMANAH_TYPE_DEFINITION);
+	storage_manager_signals[SIGNAL_DEFINITION_MODIFIED] = g_signal_new ("definition-modified",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				0, NULL, NULL,
+				g_cclosure_marshal_VOID__OBJECT,
+				G_TYPE_NONE, 1, ALMANAH_TYPE_DEFINITION);
 	storage_manager_signals[SIGNAL_DEFINITION_REMOVED] = g_signal_new ("definition-removed",
 				G_TYPE_FROM_CLASS (klass),
 				G_SIGNAL_RUN_LAST,
@@ -119,12 +128,25 @@ almanah_storage_manager_init (AlmanahStorageManager *self)
  *
  * Creates a new #AlmanahStorageManager, connected to the given database @filename.
  *
+ * If @filename is for an encrypted database, it will automatically be changed to the canonical filename for the
+ * unencrypted database, even if that file doesn't exist, and even if Almanah was compiled without encryption support.
+ * Database filenames are always passed as the unencrypted filename.
+ *
  * Return value: the new #AlmanahStorageManager
  **/
 AlmanahStorageManager *
 almanah_storage_manager_new (const gchar *filename)
 {
-	return g_object_new (ALMANAH_TYPE_STORAGE_MANAGER, "filename", filename, NULL);
+	gchar *new_filename = NULL;
+	AlmanahStorageManager *sm;
+
+	if (g_str_has_suffix (filename, ENCRYPTED_SUFFIX) == TRUE)
+		filename = new_filename = g_strndup (filename, strlen (filename) - strlen (ENCRYPTED_SUFFIX));
+
+	sm = g_object_new (ALMANAH_TYPE_STORAGE_MANAGER, "filename", filename, NULL);
+	g_free (new_filename);
+
+	return sm;
 }
 
 static void
@@ -163,7 +185,7 @@ almanah_storage_manager_set_property (GObject *object, guint property_id, const
 	switch (property_id) {
 		case PROP_FILENAME:
 			priv->plain_filename = g_strdup (g_value_get_string (value));
-			priv->filename = g_strjoin (NULL, priv->plain_filename, ".encrypted", NULL);
+			priv->filename = g_strjoin (NULL, priv->plain_filename, ENCRYPTED_SUFFIX, NULL);
 			break;
 		default:
 			/* We don't have any other property... */
@@ -185,6 +207,7 @@ create_tables (AlmanahStorageManager *self)
 	const gchar *queries[] = {
 		"CREATE TABLE IF NOT EXISTS entries (year INTEGER, month INTEGER, day INTEGER, content TEXT, PRIMARY KEY (year, month, day))",
 		"CREATE TABLE IF NOT EXISTS definitions (definition_text TEXT, definition_type INTEGER, definition_value TEXT, definition_value2 TEXT, PRIMARY KEY (definition_text))",
+		"ALTER TABLE entries ADD COLUMN is_important INTEGER", /* added in 0.7.0 */
 		NULL
 	};
 
@@ -206,23 +229,31 @@ typedef struct {
 static gboolean
 prepare_gpgme (AlmanahStorageManager *self, gboolean encrypting, CipherOperation *operation, GError **error)
 {
-	gpgme_error_t gpgme_error;
+	gpgme_error_t error_gpgme;
+
+	/* Check for a minimum GPGME version (bgo#599598) */
+	if (gpgme_check_version (MIN_GPGME_VERSION) == NULL) {
+		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_BAD_VERSION,
+			     _("GPGME is not at least version %s"),
+			     MIN_GPGME_VERSION);
+		return FALSE;
+	}
 
 	/* Check OpenPGP's supported */
-	gpgme_error = gpgme_engine_check_version (GPGME_PROTOCOL_OpenPGP);
-	if (gpgme_error != GPG_ERR_NO_ERROR) {
+	error_gpgme = gpgme_engine_check_version (GPGME_PROTOCOL_OpenPGP);
+	if (error_gpgme != GPG_ERR_NO_ERROR) {
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_UNSUPPORTED,
 			     _("GPGME doesn't support OpenPGP: %s"),
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
 	/* Set up for the operation */
-	gpgme_error = gpgme_new (&(operation->context));
-	if (gpgme_error != GPG_ERR_NO_ERROR) {
+	error_gpgme = gpgme_new (&(operation->context));
+	if (error_gpgme != GPG_ERR_NO_ERROR) {
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_CREATING_CONTEXT,
 			     _("Error creating cipher context: %s"),
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
@@ -237,7 +268,7 @@ static gboolean
 open_db_files (AlmanahStorageManager *self, gboolean encrypting, CipherOperation *operation, GError **error)
 {
 	GError *io_error = NULL;
-	gpgme_error_t gpgme_error;
+	gpgme_error_t error_gpgme;
 
 	/* Open the encrypted file */
 	operation->cipher_io_channel = g_io_channel_new_file (self->priv->filename, encrypting ? "w" : "r", &io_error);
@@ -247,12 +278,12 @@ open_db_files (AlmanahStorageManager *self, gboolean encrypting, CipherOperation
 	}
 
 	/* Pass it to GPGME */
-	gpgme_error = gpgme_data_new_from_fd (&(operation->gpgme_cipher), g_io_channel_unix_get_fd (operation->cipher_io_channel));
-	if (gpgme_error != GPG_ERR_NO_ERROR) {
+	error_gpgme = gpgme_data_new_from_fd (&(operation->gpgme_cipher), g_io_channel_unix_get_fd (operation->cipher_io_channel));
+	if (error_gpgme != GPG_ERR_NO_ERROR) {
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_OPENING_FILE,
 			     _("Error opening encrypted database file \"%s\": %s"),
 			     self->priv->filename,
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
@@ -267,12 +298,12 @@ open_db_files (AlmanahStorageManager *self, gboolean encrypting, CipherOperation
 	fchmod (g_io_channel_unix_get_fd (operation->plain_io_channel), S_IRWXU);
 
 	/* Pass it to GPGME */
-	gpgme_error = gpgme_data_new_from_fd (&(operation->gpgme_plain), g_io_channel_unix_get_fd (operation->plain_io_channel));
-	if (gpgme_error != GPG_ERR_NO_ERROR) {
+	error_gpgme = gpgme_data_new_from_fd (&(operation->gpgme_plain), g_io_channel_unix_get_fd (operation->plain_io_channel));
+	if (error_gpgme != GPG_ERR_NO_ERROR) {
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_OPENING_FILE,
 			     _("Error opening plain database file \"%s\": %s"),
 			     self->priv->plain_filename,
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
@@ -295,8 +326,11 @@ cipher_operation_free (CipherOperation *operation)
 		g_io_channel_unref (operation->plain_io_channel);
 	}
 
-	gpgme_signers_clear (operation->context);
-	gpgme_release (operation->context);
+	/* We could free the operation before the context is even created (bgo#599598) */
+	if (operation->context != NULL) {
+		gpgme_signers_clear (operation->context);
+		gpgme_release (operation->context);
+	}
 
 	g_object_unref (operation->storage_manager);
 	g_free (operation);
@@ -305,12 +339,29 @@ cipher_operation_free (CipherOperation *operation)
 static gboolean
 database_idle_cb (CipherOperation *operation)
 {
-	gpgme_error_t gpgme_error;
+	AlmanahStorageManager *self = operation->storage_manager;
+	gpgme_error_t error_gpgme;
+
+	if (gpgme_wait (operation->context, &error_gpgme, FALSE) != NULL || error_gpgme != GPG_ERR_NO_ERROR) {
+		struct stat db_stat;
+		gchar *warning_message = NULL;
+
+		/* Check to see if the encrypted file is 0B in size, which isn't good.
+		 * Not much we can do about it except quit without deleting the plaintext database. */
+		g_stat (self->priv->filename, &db_stat);
+		if (g_file_test (self->priv->filename, G_FILE_TEST_IS_REGULAR) == FALSE || db_stat.st_size == 0) {
+			warning_message = g_strdup (_("The encrypted database is empty. The plain database file has been left undeleted as backup."));
+		} else if (g_unlink (self->priv->plain_filename) != 0) {
+			/* Delete the plain file */
+			warning_message = g_strdup_printf (_("Could not delete plain database file \"%s\"."), self->priv->plain_filename);
+		}
 
-	if (!(gpgme_wait (operation->context, &gpgme_error, FALSE) == NULL && gpgme_error == 0)) {
 		/* A slight assumption that we're disconnecting at this point (we're technically
 		 * only encrypting), but a valid one. */
-		g_signal_emit (operation->storage_manager, storage_manager_signals[SIGNAL_DISCONNECTED], 0, operation->storage_manager);
+		g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0,
+			       (error_gpgme == GPG_ERR_NO_ERROR) ? NULL: gpgme_strerror (error_gpgme),
+			       warning_message);
+		g_free (warning_message);
 
 		/* Finished! */
 		cipher_operation_free (operation);
@@ -326,7 +377,7 @@ decrypt_database (AlmanahStorageManager *self, GError **error)
 {
 	GError *preparation_error = NULL;
 	CipherOperation *operation;
-	gpgme_error_t gpgme_error;
+	gpgme_error_t error_gpgme;
 
 	operation = g_new0 (CipherOperation, 1);
 	operation->storage_manager = g_object_ref (self);
@@ -340,12 +391,12 @@ decrypt_database (AlmanahStorageManager *self, GError **error)
 	}
 
 	/* Decrypt and verify! */
-	gpgme_error = gpgme_op_decrypt_verify (operation->context, operation->gpgme_cipher, operation->gpgme_plain);
-	if (gpgme_error != GPG_ERR_NO_ERROR) {
+	error_gpgme = gpgme_op_decrypt_verify (operation->context, operation->gpgme_cipher, operation->gpgme_plain);
+	if (error_gpgme != GPG_ERR_NO_ERROR) {
 		cipher_operation_free (operation);
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_DECRYPTING,
 			     _("Error decrypting database: %s"),
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
@@ -360,7 +411,7 @@ encrypt_database (AlmanahStorageManager *self, const gchar *encryption_key, GErr
 {
 	GError *preparation_error = NULL;
 	CipherOperation *operation;
-	gpgme_error_t gpgme_error;
+	gpgme_error_t error_gpgme;
 	gpgme_key_t gpgme_keys[2] = { NULL, };
 
 	operation = g_new0 (CipherOperation, 1);
@@ -374,12 +425,12 @@ encrypt_database (AlmanahStorageManager *self, const gchar *encryption_key, GErr
 	}
 
 	/* Set up signing and the recipient */
-	gpgme_error = gpgme_get_key (operation->context, encryption_key, &gpgme_keys[0], FALSE);
-	if (gpgme_error != GPG_ERR_NO_ERROR || gpgme_keys[0] == NULL) {
+	error_gpgme = gpgme_get_key (operation->context, encryption_key, &gpgme_keys[0], FALSE);
+	if (error_gpgme != GPG_ERR_NO_ERROR || gpgme_keys[0] == NULL) {
 		cipher_operation_free (operation);
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_GETTING_KEY,
 			     _("Error getting encryption key: %s"),
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
@@ -392,29 +443,21 @@ encrypt_database (AlmanahStorageManager *self, const gchar *encryption_key, GErr
 	}
 
 	/* Encrypt and sign! */
-	gpgme_error = gpgme_op_encrypt_sign_start (operation->context, gpgme_keys, 0, operation->gpgme_plain, operation->gpgme_cipher);
+	error_gpgme = gpgme_op_encrypt_sign_start (operation->context, gpgme_keys, 0, operation->gpgme_plain, operation->gpgme_cipher);
 	gpgme_key_unref (gpgme_keys[0]);
 
-	if (gpgme_error != GPG_ERR_NO_ERROR) {
+	if (error_gpgme != GPG_ERR_NO_ERROR) {
 		cipher_operation_free (operation);
 
 		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_ENCRYPTING,
 			     _("Error encrypting database: %s"),
-			     gpgme_strerror (gpgme_error));
+			     gpgme_strerror (error_gpgme));
 		return FALSE;
 	}
 
 	/* The operation will be completed in the idle function */
 	g_idle_add ((GSourceFunc) database_idle_cb, operation);
 
-	/* Delete the plain file and wait for the idle function to quit us */
-	if (g_unlink (self->priv->plain_filename) != 0) {
-		g_set_error (error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_ENCRYPTING,
-			     _("Could not delete plain database file \"%s\"."),
-			     self->priv->plain_filename);
-		return FALSE;
-	}
-
 	return TRUE;
 }
 
@@ -448,6 +491,24 @@ get_encryption_key (void)
 }
 #endif /* ENABLE_ENCRYPTION */
 
+static void
+back_up_file (const gchar *filename)
+{
+	GFile *original_file, *backup_file;
+	gchar *backup_filename;
+
+	/* Make a backup of the encrypted database file */
+	original_file = g_file_new_for_path (filename);
+	backup_filename = g_strdup_printf ("%s~", filename);
+	backup_file = g_file_new_for_path (backup_filename);
+	g_free (backup_filename);
+
+	g_file_copy_async (original_file, backup_file, G_FILE_COPY_OVERWRITE, G_PRIORITY_DEFAULT, NULL, NULL, NULL, NULL, NULL);
+
+	g_object_unref (original_file);
+	g_object_unref (backup_file);
+}
+
 gboolean
 almanah_storage_manager_connect (AlmanahStorageManager *self, GError **error)
 {
@@ -461,6 +522,9 @@ almanah_storage_manager_connect (AlmanahStorageManager *self, GError **error)
 	if (g_file_test (self->priv->filename, G_FILE_TEST_IS_REGULAR) == TRUE && encrypted_db_stat.st_size > 0) {
 		GError *child_error = NULL;
 
+		/* Make a backup of the encrypted database file */
+		back_up_file (self->priv->filename);
+
 		g_stat (self->priv->plain_filename, &plaintext_db_stat);
 
 		/* Only decrypt the database if the plaintext database doesn't exist or is empty. If the plaintext
@@ -478,12 +542,12 @@ almanah_storage_manager_connect (AlmanahStorageManager *self, GError **error)
 				g_error_free (child_error);
 			}
 		}
-
-		
 	}
 
 	self->priv->decrypted = TRUE;
 #else
+	/* Make a backup of the plaintext database file */
+	back_up_file (self->priv->plain_filename);
 	self->priv->decrypted = FALSE;
 #endif /* ENABLE_ENCRYPTION */
 
@@ -513,39 +577,42 @@ almanah_storage_manager_disconnect (AlmanahStorageManager *self, GError **error)
 	/* Close the DB connection */
 	sqlite3_close (self->priv->connection);
 
-	if (self->priv->decrypted == FALSE) {
-		g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0, self);
-		return TRUE;
-	}
-
 #ifdef ENABLE_ENCRYPTION
+	/* If the database wasn't encrypted before we opened it, we won't encrypt it when closing.
+	 * In fact, we'll go so far as to delete the old encrypted database file. */
+	if (self->priv->decrypted == FALSE) 
+		goto delete_encrypted_db;
+
+	/* Get the encryption key */
 	encryption_key = get_encryption_key ();
-	if (encryption_key == NULL) {
-		g_message (_("Error getting encryption key: GConf key \"%s\" invalid or empty. Your diary will not be encrypted; please install Seahorse and set up a default key, or ignore this message."), ENCRYPTION_KEY_GCONF_PATH);
-		g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0, self);
-		return TRUE;
-	}
+	if (encryption_key == NULL)
+		goto delete_encrypted_db;
 
 	/* Encrypt the plain DB file */
 	if (encrypt_database (self, encryption_key, &child_error) != TRUE) {
-		if (child_error->code != ALMANAH_STORAGE_MANAGER_ERROR_GETTING_KEY) {
-			/* Propagate the error */
+		g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0, NULL, child_error->message);
+
+		if (g_error_matches (child_error, ALMANAH_STORAGE_MANAGER_ERROR, ALMANAH_STORAGE_MANAGER_ERROR_GETTING_KEY) == TRUE)
 			g_propagate_error (error, child_error);
-			return FALSE;
-		}
+		else
+			g_error_free (child_error);
 
-		/* Log an error about being unable to get the key
-		 * then continue without encrypting. */
-		g_warning ("%s", child_error->message);
-		g_error_free (child_error);
-
-		g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0, self);
+		g_free (encryption_key);
+		return FALSE;
 	}
 
 	g_free (encryption_key);
-#endif /* ENABLE_ENCRYPTION */
+#else /* ENABLE_ENCRYPTION */
+	g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0, NULL, NULL);
+#endif /* !ENABLE_ENCRYPTION */
 
 	return TRUE;
+
+delete_encrypted_db:
+	/* Delete the old encrypted database and return */
+	g_unlink (self->priv->filename);
+	g_signal_emit (self, storage_manager_signals[SIGNAL_DISCONNECTED], 0, NULL, NULL);
+	return TRUE;
 }
 
 AlmanahQueryResults *
@@ -697,7 +764,7 @@ almanah_storage_manager_get_entry (AlmanahStorageManager *self, GDate *date)
 
 	/* Prepare the statement */
 	if (sqlite3_prepare_v2 (self->priv->connection,
-				"SELECT content FROM entries WHERE year = ? AND month = ? AND day = ?", -1,
+				"SELECT content, is_important FROM entries WHERE year = ? AND month = ? AND day = ?", -1,
 				&statement, NULL) != SQLITE_OK) {
 		return NULL;
 	}
@@ -716,6 +783,7 @@ almanah_storage_manager_get_entry (AlmanahStorageManager *self, GDate *date)
 	/* Get the data */
 	entry = almanah_entry_new (date);
 	almanah_entry_set_data (entry, sqlite3_column_blob (statement, 0), sqlite3_column_bytes (statement, 0));
+	almanah_entry_set_is_important (entry, (sqlite3_column_int (statement, 1) == 1) ? TRUE : FALSE);
 
 	sqlite3_finalize (statement);
 
@@ -760,7 +828,7 @@ almanah_storage_manager_set_entry (AlmanahStorageManager *self, AlmanahEntry *en
 
 		/* Prepare the statement */
 		if (sqlite3_prepare_v2 (self->priv->connection,
-					"REPLACE INTO entries (year, month, day, content) VALUES (?, ?, ?, ?)", -1,
+					"REPLACE INTO entries (year, month, day, content, is_important) VALUES (?, ?, ?, ?, ?)", -1,
 					&statement, NULL) != SQLITE_OK) {
 			return FALSE;
 		}
@@ -773,6 +841,8 @@ almanah_storage_manager_set_entry (AlmanahStorageManager *self, AlmanahEntry *en
 		data = almanah_entry_get_data (entry, &length);
 		sqlite3_bind_blob (statement, 4, data, length, SQLITE_TRANSIENT);
 
+		sqlite3_bind_int (statement, 5, almanah_entry_is_important (entry));
+
 		/* Execute the statement */
 		if (sqlite3_step (statement) != SQLITE_DONE) {
 			sqlite3_finalize (statement);
@@ -809,7 +879,7 @@ almanah_storage_manager_search_entries (AlmanahStorageManager *self, const gchar
 
 	/* Prepare the statement */
 	if (sqlite3_prepare_v2 (self->priv->connection,
-				"SELECT content, day, month, year FROM entries", -1,
+				"SELECT content, day, month, year, is_important FROM entries", -1,
 				&statement, NULL) != SQLITE_OK) {
 		return -1;
 	}
@@ -826,6 +896,7 @@ almanah_storage_manager_search_entries (AlmanahStorageManager *self, const gchar
 		g_date_set_dmy (date, sqlite3_column_int (statement, 1), sqlite3_column_int (statement, 2), sqlite3_column_int (statement, 3));
 		entry = almanah_entry_new (date);
 		almanah_entry_set_data (entry, sqlite3_column_blob (statement, 0), sqlite3_column_bytes (statement, 0));
+		almanah_entry_set_is_important (entry, (sqlite3_column_int (statement, 4) == 1) ? TRUE : FALSE);
 
 		/* Deserialise the entry into our buffer */
 		gtk_text_buffer_set_text (text_buffer, "", 0);
@@ -853,12 +924,67 @@ almanah_storage_manager_search_entries (AlmanahStorageManager *self, const gchar
 	return result_count - 1;
 }
 
+/**
+ * almanah_storage_manager_get_entries:
+ * @self: an #AlmanahStorageManager
+ *
+ * Returns a list of all #AlmanahEntry<!-- -->s in the database.
+ *
+ * Return value: a #GSList of #AlmanahEntry<!-- -->s, or %NULL; unref elements with g_object_unref(); free list with g_slist_free()
+ **/
+GSList *
+almanah_storage_manager_get_entries (AlmanahStorageManager *self)
+{
+	GSList *entries = NULL;
+	int result;
+	sqlite3_stmt *statement;
+
+	/* Just as with almanah_storage_manager_get_entry(), it's necessary to avoid our nice SQLite interface
+	 * here. It's probably more efficient to avoid it anyway. */
+
+	/* Prepare the statement */
+	if (sqlite3_prepare_v2 (self->priv->connection,
+				"SELECT content, is_important, day, month, year FROM entries", -1,
+				&statement, NULL) != SQLITE_OK) {
+		return NULL;
+	}
+
+	/* Execute the statement */
+	while ((result = sqlite3_step (statement)) == SQLITE_ROW) {
+		GDate date;
+		AlmanahEntry *entry;
+
+		g_date_set_dmy (&date,
+				sqlite3_column_int (statement, 2),
+				sqlite3_column_int (statement, 3),
+				sqlite3_column_int (statement, 4));
+
+		/* Get the data */
+		entry = almanah_entry_new (&date);
+		almanah_entry_set_data (entry, sqlite3_column_blob (statement, 0), sqlite3_column_bytes (statement, 0));
+		almanah_entry_set_is_important (entry, (sqlite3_column_int (statement, 1) == 1) ? TRUE : FALSE);
+
+		entries = g_slist_prepend (entries, entry);
+	}
+
+	sqlite3_finalize (statement);
+
+	/* Check for errors */
+	if (result != SQLITE_DONE) {
+		g_slist_foreach (entries, (GFunc) g_object_unref, NULL);
+		g_slist_free (entries);
+		return NULL;
+	}
+
+	return g_slist_reverse (entries);
+}
+
 /* NOTE: Free results with g_free. Return value is 0-based. */
 gboolean *
 almanah_storage_manager_get_month_marked_days (AlmanahStorageManager *self, GDateYear year, GDateMonth month, guint *num_days)
 {
 	AlmanahQueryResults *results;
-	guint i;
+	gint i;
 	gboolean *days;
 
 	i = g_date_get_days_in_month (month, year);
@@ -885,55 +1011,57 @@ almanah_storage_manager_get_month_marked_days (AlmanahStorageManager *self, GDat
  * almanah_storage_manager_get_definitions:
  * @self: an #AlmanahStorageManager
  *
- * Returns a %NULL-terminated array of all the #AlmanahDefinitions in the
- * database. Each #AlmanahDefinition should be unreffed, and the array should
- * be freed with g_free().
+ * Returns a list of all #AlmanahDefinition<!-- -->s in the database.
  *
- * On error, an array with a single %NULL element will be returned. The array
- * should still be freed with g_free().
- *
- * Return value: a %NULL-terminated array of #AlmanahDefinitions
+ * Return value: a #GSList of #AlmanahDefinition<!-- -->s, or %NULL; unref elements with g_object_unref(); free list with g_slist_free()
  **/
-AlmanahDefinition **
+GSList *
 almanah_storage_manager_get_definitions (AlmanahStorageManager *self)
 {
-	AlmanahQueryResults *results;
-	AlmanahDefinition **definitions;
-	guint i;
+	GSList *definitions = NULL;
+	int result;
+	sqlite3_stmt *statement;
 
-	results = almanah_storage_manager_query (self, "SELECT definition_type, definition_value, definition_value2, definition_text FROM definitions", NULL);
+	/* It's more efficient to avoid our nice SQLite interface and do things manually. */
 
-	if (results == NULL || results->rows == 0) {
-		if (results != NULL)
-			almanah_storage_manager_free_results (results);
-
-		/* Return empty array */
-		definitions = (AlmanahDefinition**) g_new (AlmanahDefinition*, 1);
-		definitions[0] = NULL;
-		return definitions;
+	/* Prepare the statement */
+	if (sqlite3_prepare_v2 (self->priv->connection,
+				"SELECT definition_type, definition_value, definition_value2, definition_text FROM definitions", -1,
+				&statement, NULL) != SQLITE_OK) {
+		return NULL;
 	}
 
-	definitions = (AlmanahDefinition**) g_new (AlmanahDefinition*, results->rows + 1);
-	for (i = 0; i < results->rows; i++) {
-		definitions[i] = almanah_definition_new (atoi (results->data[(i + 1) * results->columns]));
-		almanah_definition_set_value (definitions[i], results->data[(i + 1) * results->columns + 1]);
-		almanah_definition_set_value2 (definitions[i], results->data[(i + 1) * results->columns + 2]);
-		almanah_definition_set_text (definitions[i], results->data[(i + 1) * results->columns + 3]);
+	/* Execute the statement */
+	while ((result = sqlite3_step (statement)) == SQLITE_ROW) {
+		AlmanahDefinition *definition = almanah_definition_new (sqlite3_column_int (statement, 0));
+		almanah_definition_set_value (definition, (gchar*) sqlite3_column_text (statement, 1));
+		almanah_definition_set_value2 (definition, (gchar*) sqlite3_column_text (statement, 2));
+		almanah_definition_set_text (definition, (gchar*) sqlite3_column_text (statement, 3));
+
+		definitions = g_slist_prepend (definitions, definition);
 	}
-	definitions[i] = NULL;
 
-	almanah_storage_manager_free_results (results);
+	sqlite3_finalize (statement);
 
-	return definitions;
+	/* Check for errors */
+	if (result != SQLITE_DONE) {
+		g_slist_foreach (definitions, (GFunc) g_object_unref, NULL);
+		g_slist_free (definitions);
+		return NULL;
+	}
+
+	return g_slist_reverse (definitions);
 }
 
+/* Note: this function is case-insensitive, unless the definition text contains Unicode characters
+ * beyond the ASCII range. This is an SQLite bug: http://sqlite.org/lang_expr.html#like */
 AlmanahDefinition *
 almanah_storage_manager_get_definition (AlmanahStorageManager *self, const gchar *definition_text)
 {
 	AlmanahQueryResults *results;
 	AlmanahDefinition *definition;
 
-	results = almanah_storage_manager_query (self, "SELECT definition_type, definition_value, definition_value2, definition_text FROM definitions WHERE definition_text = '%q' LIMIT 1", NULL,
+	results = almanah_storage_manager_query (self, "SELECT definition_type, definition_value, definition_value2, definition_text FROM definitions WHERE definition_text LIKE '%q' LIMIT 1", NULL,
 						 definition_text);
 
 	if (results == NULL || results->rows == 0) {
@@ -958,12 +1086,22 @@ almanah_storage_manager_add_definition (AlmanahStorageManager *self, AlmanahDefi
 	gboolean return_value;
 	const gchar *value, *value2, *text;
 	AlmanahDefinitionType type_id;
+	AlmanahDefinition *real_definition;
 
 	type_id = almanah_definition_get_type_id (definition);
 	value = almanah_definition_get_value (definition);
 	value2 = almanah_definition_get_value2 (definition);
 	text = almanah_definition_get_text (definition);
 
+	/* Check to see if there's already a definition for this text (case-insensitively), and
+	 * use its (correctly-cased) definition text instead of ours */
+	real_definition = almanah_storage_manager_get_definition (self, text);
+	if (real_definition != NULL) {
+		text = almanah_definition_get_text (real_definition);
+		almanah_definition_set_text (definition, text);
+	}
+
+	/* Update/Insert the definition */
 	if (value2 == NULL) {
 		return_value = almanah_storage_manager_query_async (self, "REPLACE INTO definitions (definition_type, definition_value, definition_text) VALUES (%u, '%q', '%q')", NULL, NULL, NULL,
 								    type_id,
@@ -977,7 +1115,12 @@ almanah_storage_manager_add_definition (AlmanahStorageManager *self, AlmanahDefi
 								    text);
 	}
 
-	if (return_value == TRUE)
+	if (real_definition != NULL)
+		g_object_unref (real_definition);
+
+	if (return_value == TRUE && real_definition != NULL)
+		g_signal_emit (self, storage_manager_signals[SIGNAL_DEFINITION_MODIFIED], 0, definition);
+	else if (return_value == TRUE)
 		g_signal_emit (self, storage_manager_signals[SIGNAL_DEFINITION_ADDED], 0, definition);
 
 	return return_value;
